@@ -3,7 +3,7 @@
  * command.h
  *	  a list of commands and translations between keys and commands
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/commands.h
@@ -64,9 +64,12 @@ typedef enum PspgCommand
 	cmd_SetTheme_SolarDark,
 	cmd_SetTheme_SolarLight,
 	cmd_SetTheme_GruvboxLight,
+	cmd_SetTheme_TaoLight,
 	cmd_SaveSetup,
 
+	cmd_Escape,
 	cmd_Quit,
+	cmd_RawOutputQuit,
 	cmd_ShowMenu,
 	cmd_FlushBookmarks,
 	cmd_ToggleBookmark,
@@ -94,18 +97,26 @@ typedef enum PspgCommand
 	cmd_BackwardSearch,
 	cmd_SearchNext,
 	cmd_SearchPrev,
+	cmd_SearchColumn,
 	cmd_ShowTopBar,
 	cmd_ShowBottomBar,
 	cmd_RowNumToggle,
 	cmd_GotoLine,
-	cmd_ShowCursor
+	cmd_ShowCursor,
+	cmd_ShowVerticalCursor,
+	cmd_BoldLabelsToggle,
+	cmd_BoldCursorToggle,
+	cmd_SortAsc,
+	cmd_SortDesc,
+	cmd_OriginalSort,
+	cmd_TogglePause
 } PspgCommand;
 
 extern void initialize_special_keycodes();
 extern const char *cmd_string(int cmd);
-extern int translate_event(int c, bool alt);
+extern int translate_event(int c, bool alt, bool watch_mode);
 
 extern int cmd_get_theme(int cmd);
 extern int theme_get_cmd(int theme);
 
-#endif
\ No newline at end of file
+#endif
