@@ -3,7 +3,7 @@
  * config.c
  *	  a routines for loading, saving configuration
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/config.c
@@ -72,34 +72,55 @@ parse_cfg(char *line, char *key, bool *bool_val, int *int_val)
 	return false;
 }
 
+#define SAFE_SAVE_BOOL_OPTION(name, opt)		\
+do { \
+	result = fprintf(f, "%s = %s\n", (name), (opt) ? "true" : "false"); \
+	if (result < 0) \
+		return false; \
+} while (0)
+
 bool
 save_config(char *path, Options *opts)
 {
-	FILE *f;
+	FILE	   *f;
+	int			result;
 
 	errno = 0;
 	f = fopen(path, "w");
 	if (f == NULL)
 		return false;
 
-	fprintf(f, "ascii_menu = %s\n", opts->force_ascii_art ? "true" : "false");
-	fprintf(f, "bold_labels = %s\n", opts->bold_labels ? "true" : "false");
-	fprintf(f, "bold_cursor = %s\n", opts->bold_cursor ? "true" : "false");
-	fprintf(f, "ignore_case = %s\n", opts->ignore_case ? "true" : "false");
-	fprintf(f, "ignore_lower_case = %s\n", opts->ignore_lower_case ? "true" : "false");
-	fprintf(f, "no_cursor = %s\n", opts->no_cursor ? "true" : "false");
-	fprintf(f, "no_sound = %s\n", opts->no_sound ? "true" : "false");
-	fprintf(f, "no_mouse = %s\n", opts->no_mouse ? "true" : "false");
-	fprintf(f, "less_status_bar = %s\n", opts->less_status_bar ? "true" : "false");
-	fprintf(f, "no_highlight_search = %s\n", opts->no_highlight_search ? "true" : "false");
-	fprintf(f, "no_highlight_lines = %s\n", opts->no_highlight_lines ? "true" : "false");
-	fprintf(f, "force_uniborder = %s\n", opts->force_uniborder ? "true" : "false");
-	fprintf(f, "show_rownum = %s\n", opts->show_rownum ? "true" : "false");
-	fprintf(f, "theme = %d\n", opts->theme);
-	fprintf(f, "without_commandbar = %s\n", opts->no_commandbar ? "true" : "false");
-	fprintf(f, "without_topbar = %s\n", opts->no_topbar ? "true" : "false");
+	SAFE_SAVE_BOOL_OPTION("ascii_menu", opts->force_ascii_art);
+	SAFE_SAVE_BOOL_OPTION("bold_labels", opts->bold_labels);
+	SAFE_SAVE_BOOL_OPTION("bold_cursor", opts->bold_cursor);
+	SAFE_SAVE_BOOL_OPTION("ignore_case", opts->ignore_case);
+	SAFE_SAVE_BOOL_OPTION("ignore_lower_case", opts->ignore_lower_case);
+	SAFE_SAVE_BOOL_OPTION("no_cursor", opts->no_cursor);
+	SAFE_SAVE_BOOL_OPTION("no_sound", opts->no_sound);
+	SAFE_SAVE_BOOL_OPTION("no_mouse", opts->no_mouse);
+	SAFE_SAVE_BOOL_OPTION("less_status_bar", opts->less_status_bar);
+	SAFE_SAVE_BOOL_OPTION("no_highlight_search", opts->no_highlight_search);
+	SAFE_SAVE_BOOL_OPTION("no_highlight_lines", opts->no_highlight_lines);
+	SAFE_SAVE_BOOL_OPTION("force_uniborder", opts->force_uniborder);
+	SAFE_SAVE_BOOL_OPTION("show_rownum", opts->show_rownum);
+	SAFE_SAVE_BOOL_OPTION("without_commandbar", opts->no_commandbar);
+	SAFE_SAVE_BOOL_OPTION("without_topbar", opts->no_topbar);
+	SAFE_SAVE_BOOL_OPTION("vertical_cursor", opts->vertical_cursor);
+	SAFE_SAVE_BOOL_OPTION("on_sigint_exit", opts->on_sigint_exit);
+	SAFE_SAVE_BOOL_OPTION("no_sigint_search_reset", opts->no_sigint_search_reset);
+	SAFE_SAVE_BOOL_OPTION("double_header", opts->double_header);
 
-	fclose(f);
+	result = fprintf(f, "theme = %d\n", opts->theme);
+	if (result < 0)
+		return false;
+
+	result = fprintf(f, "border_type = %d\n", opts->border_type);
+	if (result < 0)
+		return false;
+
+	result = fclose(f);
+	if (result != 0)
+		return false;
 
 	return true;
 }
@@ -161,6 +182,16 @@ load_config(char *path, Options *opts)
 				opts->no_commandbar = bool_val;
 			else if (strcmp(key, "without_topbar") == 0)
 				opts->no_topbar = bool_val;
+			else if (strcmp(key, "vertical_cursor") == 0)
+				opts->vertical_cursor = bool_val;
+			else if (strcmp(key, "border_type") == 0)
+				opts->border_type = int_val;
+			else if (strcmp(key, "double_header") == 0)
+				opts->double_header = bool_val;
+			else if (strcmp(key, "on_sigint_exit") == 0)
+				opts->on_sigint_exit = bool_val;
+			else if (strcmp(key, "no_sigint_search_reset") == 0)
+				opts->no_sigint_search_reset = bool_val;
 
 			free(line);
 			line = NULL;
