@@ -3,7 +3,7 @@
  * config.h
  *	  load/save configuration
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/config.h
@@ -32,12 +32,31 @@ typedef struct
 	bool	no_topbar;
 	bool	show_rownum;
 	bool	no_cursor;
+	bool	vertical_cursor;
 	bool	tabular_cursor;
 	bool	force_ascii_art;
 	int		theme;
 	int		freezed_cols;
 	bool	bold_labels;
 	bool	bold_cursor;
+	bool	tsv_format;
+	bool	csv_format;
+	char	csv_separator;
+	char	csv_header;			/* a - auto, - off, + on */
+	char   *nullstr;
+	bool	ignore_short_rows;
+	char	double_header;
+	int		border_type;
+	bool	on_sigint_exit;
+	bool	no_sigint_search_reset;
+	char   *query;
+	int		watch_time;
+	char   *host;
+	char   *username;
+	char   *port;
+	bool	force_password_prompt;
+	char   *password;
+	char   *dbname;
 } Options;
 
 extern bool save_config(char *path, Options *opts);
