@@ -3,7 +3,7 @@
  * menu.c
  *	 holds menu related code
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/menu.c
@@ -21,6 +21,8 @@
 #define MENU_ITEM_OPTIONS			2
 
 ST_CMDBAR_ITEM _bottombar[] = {
+	{"Save", false, 2, cmd_SaveData},
+	{"Search", false, 7, cmd_ForwardSearch},
 	{"Menu", false, 9, cmd_ShowMenu},
 	{"Quit", false, 10, cmd_Quit},
 	{NULL}
@@ -29,7 +31,7 @@ ST_CMDBAR_ITEM _bottombar[] = {
 ST_MENU_ITEM _file[] = {
 	{"~S~ave", cmd_SaveData, "s"},
 	{"--"},
-	{"Raw output ~q~uit", cmd_RawOutputQuit, "M-q"},
+	{"~R~aw output quit", cmd_RawOutputQuit, "M-q"},
 	{"E~x~it", cmd_Quit, "q, F10"},
 	{NULL}
 };
@@ -44,6 +46,8 @@ ST_MENU_ITEM _search[] = {
 	{"Search ~a~gain", cmd_SearchNext, "n"},
 	{"Search p~r~evious", cmd_SearchPrev, "N"},
 	{"--"},
+	{"Search ~c~olumn", cmd_SearchColumn, "c"},
+	{"--"},
 	{"~T~oggle bookmark", cmd_ToggleBookmark, "M-k"},
 	{"~P~rev bookmark", cmd_PrevBookmark, "M-i"},
 	{"~N~ext bookmark", cmd_NextBookmark, "M-j"},
@@ -72,6 +76,10 @@ ST_MENU_ITEM _command[] = {
 	{"--"},
 	{"Page up", cmd_PageUp, "C-b, Prev page"},
 	{"Page down", cmd_PageDown, "C-f, space, Next page"},
+	{"--"},
+	{"As~c~ending order", cmd_SortAsc, "a"},
+	{"~D~escending order", cmd_SortDesc, "d"},
+	{"~O~riginal order", cmd_OriginalSort, "u"},
 	{NULL}
 };
 
@@ -110,6 +118,7 @@ ST_MENU_ITEM _options[] = {
 	{"~W~ithout highlighting", cmd_NoHighlight, NULL},
 	{"--"},
 	{"Show cursor", cmd_ShowCursor, "M-c"},
+	{"Show vertical cursor", cmd_ShowVerticalCursor, "M-v"},
 	{"Show line ~n~umbers", cmd_RowNumToggle, "M-n"},
 	{"Show top bar", cmd_ShowTopBar, NULL},
 	{"Show bottom bar", cmd_ShowBottomBar, NULL},
@@ -170,7 +179,7 @@ get_menu_style(int main_theme)
 		case 9:
 			return ST_MENU_STYLE_PERFECT;
 		case 10:
-			return ST_MENU_STYLE_VISION;
+			return ST_MENU_STYLE_XGOLD_BLACK;
 		case 11:
 			return ST_MENU_STYLE_OLD_TURBO;
 		case 12:
@@ -183,6 +192,8 @@ get_menu_style(int main_theme)
 			return ST_MENU_STYLE_PERFECT;
 		case 16:
 			return ST_MENU_STYLE_ONECOLOR;
+		case 20:
+			return ST_MENU_STYLE_TAO;
 		default:
 			return ST_MENU_STYLE_VISION;
 	}
@@ -194,6 +205,8 @@ get_menu_style(int main_theme)
 void
 init_menu_config(Options *opts)
 {
+	int start_from_rgb = 220;
+
 	menu_config.force8bit = opts->force8bit;
 	menu_config.force_ascii_art = opts->force_ascii_art;
 	menu_config.language = NULL;
@@ -212,8 +225,8 @@ init_menu_config(Options *opts)
 		fcp = st_menu_load_style(&menu_config, menu_theme, 30);
 		st_menu_load_style(&menu_config2, ST_MENU_STYLE_FREE_DOS_P, fcp);
 	}
-		st_menu_load_style(&menu_config, menu_theme,
-								menu_theme == ST_MENU_STYLE_ONECOLOR ? 3 : 30);
+		st_menu_load_style_rgb(&menu_config, menu_theme,
+								menu_theme == ST_MENU_STYLE_ONECOLOR ? 3 : 30, &start_from_rgb);
 
 	if (opts->theme == 1)
 		menu_config.shadow_width = 2;
@@ -296,6 +309,7 @@ post_menu(Options *opts, struct ST_MENU *menu)
 
 	st_menu_set_option(menu, cmd_RowNumToggle, ST_MENU_OPTION_MARKED, opts->show_rownum);
 	st_menu_set_option(menu, cmd_ShowCursor, ST_MENU_OPTION_MARKED, !opts->no_cursor);
+	st_menu_set_option(menu, cmd_ShowVerticalCursor, ST_MENU_OPTION_MARKED, opts->vertical_cursor);
 
 	st_menu_set_option(menu, cmd_BoldLabelsToggle, ST_MENU_OPTION_MARKED, opts->bold_labels);
 	st_menu_set_option(menu, cmd_BoldCursorToggle, ST_MENU_OPTION_MARKED, opts->bold_cursor);
