@@ -3,7 +3,7 @@
  * print.c
  *	  visualisation loaded data
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/print.c
@@ -11,19 +11,132 @@
  *-------------------------------------------------------------------------
  */
 
+#if defined HAVE_NCURSESW_CURSES_H
+#include <ncursesw/curses.h>
+#elif defined HAVE_NCURSESW_H
+#include <ncursesw.h>
+#elif defined HAVE_NCURSES_CURSES_H
+#include <ncurses/curses.h>
+#elif defined HAVE_NCURSES_H
+#include <ncurses.h>
+#elif defined HAVE_CURSES_H
 #include <curses.h>
+#else
+/* fallback */
+#include <ncurses/ncurses.h>
+#endif
+
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <sys/ioctl.h>
 
+#ifndef GWINSZ_IN_SYS_IOCTL
+#include <termios.h>
+#endif
+
+#include <unistd.h>
+
 #include "pspg.h"
 #include "unicode.h"
 
+/*
+ * Flush data to window. When row is a decoration, then
+ * replace ascii decoration by special terminal decoration.
+ */
+static void
+flush_bytes(WINDOW *win,
+		    char *rowstr,
+		    int bytes,
+		    int offsetx,
+		    bool is_top_deco,
+		    bool is_head_deco,
+		    bool is_bottom_deco,
+			DataDesc *desc,
+			ScrDesc *scrdesc,
+			Options *opts)
+{
+	if ((is_top_deco || is_head_deco || is_bottom_deco) &&
+		desc->linestyle == 'a' && opts->force_uniborder)
+	{
+		while (bytes > 0)
+		{
+			int		column_format = desc->headline_transl[offsetx];
+
+			if (column_format == 'd' && *rowstr == '-')
+			{
+				waddch(win, ACS_HLINE);
+				rowstr += 1;
+				bytes -= 1;
+				offsetx += 1;
+			}
+			else if (column_format == 'L' && (*rowstr == '+' || *rowstr == '|'))
+			{
+				if (is_head_deco)
+					waddch(win, ACS_LTEE);
+				else if (is_top_deco)
+					waddch(win, ACS_ULCORNER);
+				else /* bottom row */
+					waddch(win, ACS_LLCORNER);
+
+				rowstr += 1;
+				bytes -= 1;
+				offsetx += 1;
+			}
+			else if (column_format == 'I' && *rowstr == '+')
+			{
+				if (is_head_deco)
+					waddch(win, ACS_PLUS);
+				else if (is_top_deco)
+					waddch(win, ACS_TTEE);
+				else /* bottom row */
+					waddch(win, ACS_BTEE);
+
+				rowstr += 1;
+				bytes -= 1;
+				offsetx += 1;
+			}
+			else if (column_format == 'R' && (*rowstr == '+' || *rowstr == '|'))
+			{
+				if (is_head_deco)
+					waddch(win, ACS_RTEE);
+				else if (is_top_deco)
+					waddch(win, ACS_URCORNER);
+				else /* bottom row */
+					waddch(win, ACS_LRCORNER);
+
+				rowstr += 1;
+				bytes -= 1;
+				offsetx += 1;
+			}
+			else
+			{
+				int len = opts->force8bit ? 1 : utf8charlen(*rowstr);
+
+				waddnstr(win, rowstr, len);
+				offsetx += utf_dsplen(rowstr);
+				rowstr +=len;
+				bytes -= len;
+			}
+		}
+	}
+	else
+	{
+		/*
+		 * waddnstr is working with utf8 on ncursesw
+		 */
+		waddnstr(win, rowstr, bytes);
+	}
+}
+
 void
 window_fill(int window_identifier,
 			int srcy,
 			int srcx,						/* offset to displayed data */
 			int cursor_row,					/* row of row cursor */
+			int vcursor_xmin,				/* xmin in display coordinates */
+			int vcursor_xmax,				/* xmax in display coordinates */
 			DataDesc *desc,
 			ScrDesc *scrdesc,
 			Options *opts)
@@ -42,8 +155,6 @@ window_fill(int window_identifier,
 	bool		is_fix_rows = window_identifier == WINDOW_LUC || window_identifier == WINDOW_FIX_ROWS;
 	bool		is_rownum = window_identifier == WINDOW_ROWNUM;
 	bool		is_rownum_luc = window_identifier == WINDOW_ROWNUM_LUC;
-	int			positions[100][2];
-	int			npositions;
 
 	win = scrdesc->wins[window_identifier];
 	t = &scrdesc->themes[window_identifier];
@@ -84,42 +195,64 @@ window_fill(int window_identifier,
 	{
 		int			bytes;
 		char	   *ptr;
-		char	   *rowstr;
+		char	   *rowstr = NULL;
+		bool		line_is_valid = false;
 		LineInfo   *lineinfo = NULL;
 		bool		is_bookmark_row = false;
 		bool		is_cursor_row = false;
 		bool		is_found_row = false;
 		bool		is_pattern_row = false;
 		char		buffer[10];
+		int			positions[100][2];
+		int			npositions = 0;
 
 		is_cursor_row = (!opts->no_cursor && row == cursor_row);
 
-		if (lnb_row == 1000)
+		if (desc->order_map)
 		{
-			lnb = lnb->next;
-			lnb_row = 0;
-		}
+			int		rowno = row + srcy_bak;
 
-		if (lnb != NULL && lnb_row < lnb->nrows)
-		{
-			rowstr = lnb->rows[lnb_row];
-			if (lnb->lineinfo != NULL)
-				lineinfo = &lnb->lineinfo[lnb_row];
-			else
-				lineinfo = NULL;
-			lnb_row += 1;
-
-			/* when rownum is printed, don't process original text */
-			if (is_rownum)
+			if (rowno <= desc->last_row)
 			{
-				int rowno = row + srcy_bak + 1 - desc->first_data_row;
+				MappedLine *mp = &desc->order_map[rowno];
 
-				snprintf(buffer, sizeof(buffer), "%*d ", maxx - 1, rowno);
-				rowstr = buffer;
+				lnb = mp->lnb;
+				lnb_row = mp->lnb_row;
+				rowstr = lnb->rows[lnb_row];
+				lineinfo = lnb->lineinfo ? &lnb->lineinfo[lnb_row] : NULL;
+
+				line_is_valid = true;
 			}
 		}
 		else
-			rowstr = NULL;
+		{
+			if (lnb_row == 1000)
+			{
+				lnb = lnb ? lnb->next : NULL;
+				lnb_row = 0;
+			}
+
+			if (lnb != NULL && lnb_row < lnb->nrows)
+			{
+				rowstr = lnb->rows[lnb_row];
+				if (lnb->lineinfo != NULL)
+					lineinfo = &lnb->lineinfo[lnb_row];
+				else
+					lineinfo = NULL;
+				lnb_row += 1;
+
+				line_is_valid = true;
+			}
+		}
+
+		/* when rownum is printed, don't process original text */
+		if (is_rownum && line_is_valid)
+		{
+			int rowno = row + srcy_bak + 1 - desc->first_data_row;
+
+			snprintf(buffer, sizeof(buffer), "%*d ", maxx - 1, rowno);
+			rowstr = buffer;
+		}
 
 		is_bookmark_row = (lineinfo != NULL && (lineinfo->mask & LINEINFO_BOOKMARK) != 0) ? true : false;
 
@@ -206,8 +339,6 @@ window_fill(int window_identifier,
 		{
 			const char *str = rowstr;
 
-			npositions = 0;
-
 			while (str != NULL && npositions < 100)
 			{
 				str = pspg_search(opts, scrdesc, str);
@@ -283,12 +414,22 @@ window_fill(int window_identifier,
 
 		if (rowstr != NULL)
 		{
-			int		i;
+			int		i = 0;
 			int		effective_row = row + srcy_bak - 1;		/* row was incremented before, should be reduced */
 			bool	fix_line_attr_style;
 			bool	is_expand_head;
 			int		ei_min, ei_max;
 			int		left_spaces;							/* aux left spaces */
+			int		saved_pos;
+
+			bool	is_top_deco = false;
+			bool	is_head_deco = false;
+			bool	is_bottom_deco = false;
+			int		trailing_spaces = 0;
+
+			is_top_deco = effective_row == desc->border_top_row;
+			is_head_deco = effective_row == desc->border_head_row;
+			is_bottom_deco = effective_row == desc->border_bottom_row;
 
 			is_found_row = scrdesc->found && scrdesc->found_row == effective_row;
 
@@ -308,9 +449,11 @@ window_fill(int window_identifier,
 			{
 				if (!is_footer)
 				{
-					fix_line_attr_style = effective_row == desc->border_top_row ||
-												effective_row == desc->border_head_row ||
-												effective_row == desc->border_bottom_row;
+					is_top_deco = effective_row == desc->border_top_row;
+					is_head_deco = effective_row == desc->border_head_row;
+					is_bottom_deco = effective_row == desc->border_bottom_row;
+
+					fix_line_attr_style = is_top_deco || is_head_deco || is_bottom_deco;
 				}
 				else
 					fix_line_attr_style = false;
@@ -359,6 +502,9 @@ window_fill(int window_identifier,
 				int		aux_left_spaces = left_spaces;
 
 				free_row = malloc(left_spaces + strlen(rowstr) + 1);
+				if (!free_row)
+					leave_ncurses("out of memory");
+
 				p = free_row;
 				while (aux_left_spaces-- > 0)
 				{
@@ -372,19 +518,38 @@ window_fill(int window_identifier,
 
 			ptr = rowstr;
 			bytes = 0;
+			i = 0;
+			saved_pos = srcx;
 
 			/* find length of maxx characters */
 			if (*ptr != '\0')
 			{
-				i = 0;
 				while (i < maxx)
 				{
+					bool	is_cursor;
+					bool	is_cross_cursor = false;
+					bool	is_vertical_cursor = false;
+					int		pos = (i != -1) ? srcx + i : -1;
+					bool	skip_char = false;
+
+					if (i != -1 && vcursor_xmin <= i && i <= vcursor_xmax)
+					{
+						is_cross_cursor = is_cursor_row;
+						is_cursor = !is_cursor_row && !is_pattern_row;
+						is_vertical_cursor = true;
+					}
+					else
+					{
+						is_cross_cursor = false;
+						is_vertical_cursor = false;
+						is_cursor = is_cursor_row;
+					}
+
 					if (is_expand_head && !is_pattern_row && !is_bookmark_row)
 					{
-						int		pos = srcx + i;
-						int		new_attr;
+						attr_t		new_attr;
 
-						if (is_cursor_row)
+						if (is_cursor)
 							new_attr = pos >= ei_min && pos <= ei_max ? t->cursor_expi_attr : t->cursor_line_attr;
 						else
 							new_attr = pos >= ei_min && pos <= ei_max ? t->expi_attr : t->line_attr;
@@ -396,6 +561,7 @@ window_fill(int window_identifier,
 								waddnstr(win, rowstr, bytes);
 								rowstr += bytes;
 								bytes = 0;
+								saved_pos = pos;
 							}
 
 							/* disable current style */
@@ -408,12 +574,11 @@ window_fill(int window_identifier,
 					}
 					else if (!fix_line_attr_style)
 					{
-						int		htrpos = srcx + i;
-						int		new_attr = active_attr;
+						attr_t	new_attr = active_attr;
 						bool	print_acs_vline = false;
 						char	column_format;
 
-						column_format = desc->headline_transl != NULL ? desc->headline_transl[htrpos] : ' ';
+						column_format = desc->headline_transl != NULL && pos >= 0 ? desc->headline_transl[pos] : ' ';
 
 						if (opts->force_uniborder && desc->linestyle == 'a')
 						{
@@ -424,29 +589,35 @@ window_fill(int window_identifier,
 							}
 						}
 
-						if (is_bookmark_row)
+						if (is_cross_cursor)
+						{
+							new_attr = column_format == 'd' ? t->cross_cursor_attr : t->cross_cursor_line_attr;
+						}
+						else if (is_bookmark_row)
 						{
 							if (!is_cursor_row )
 								new_attr = column_format == 'd' ? t->bookmark_data_attr : t->bookmark_line_attr;
 							else
 								new_attr = t->cursor_bookmark_attr;
 						}
-						else if (is_pattern_row && !is_cursor_row)
+						else if (is_pattern_row && !is_cursor)
 						{
 							if (is_footer)
 								new_attr = t->pattern_data_attr;
-							else if (htrpos < desc->headline_char_size)
+							else if (is_vertical_cursor)
+								new_attr = column_format == 'd' ? t->pattern_vertical_cursor_attr : t->pattern_vertical_cursor_line_attr;
+							else if (pos < desc->headline_char_size)
 								new_attr = column_format == 'd' ? t->pattern_data_attr : t->pattern_line_attr;
 
-							if (new_attr == t->pattern_data_attr && htrpos >= lineinfo->start_char)
+							if ((new_attr == t->pattern_data_attr || new_attr == t->pattern_vertical_cursor_attr) && pos >= lineinfo->start_char)
 							{
 								if ((lineinfo->mask & LINEINFO_FOUNDSTR_MULTI) != 0)
 								{
-									int		i;
+									int		j;
 
-									for (i = 0; i < npositions; i++)
+									for (j = 0; j < npositions; j++)
 									{
-										if (htrpos >= positions[i][0] && htrpos < positions[i][1])
+										if (pos >= positions[j][0] && pos < positions[j][1])
 										{
 											new_attr = t->found_str_attr;
 											break;
@@ -455,35 +626,35 @@ window_fill(int window_identifier,
 								}
 								else
 								{
-									if (htrpos < lineinfo->start_char + scrdesc->searchterm_char_size)
+									if (pos < lineinfo->start_char + scrdesc->searchterm_char_size)
 										new_attr = t->found_str_attr;
 								}
 							}
 						}
 						else if (is_footer)
-							new_attr = is_cursor_row ? t->cursor_data_attr : t->data_attr;
-						else if (htrpos < desc->headline_char_size)
+							new_attr = is_cursor ? t->cursor_data_attr : t->data_attr;
+						else if (pos < desc->headline_char_size)
 						{
-							if (is_cursor_row )
+							if (is_cursor )
 								new_attr = column_format == 'd' ? t->cursor_data_attr : t->cursor_line_attr;
 							else
 								new_attr = column_format == 'd' ? t->data_attr : t->line_attr;
 						}
 
-						if (is_cursor_row)
+						if (is_cursor || is_cross_cursor)
 						{
-							if (is_found_row && htrpos >= scrdesc->found_start_x &&
-									htrpos < scrdesc->found_start_x + scrdesc->searchterm_char_size)
-								new_attr = new_attr ^ A_REVERSE | pattern_fix;
-							else if (is_pattern_row && htrpos >= lineinfo->start_char)
+							if (is_found_row && pos >= scrdesc->found_start_x &&
+									pos < scrdesc->found_start_x + scrdesc->searchterm_char_size)
+								new_attr = new_attr ^ ( A_REVERSE | pattern_fix );
+							else if (is_pattern_row && pos >= lineinfo->start_char)
 							{
 								if ((lineinfo->mask & LINEINFO_FOUNDSTR_MULTI) != 0)
 								{
-									int		i;
+									int		j;
 
-									for (i = 0; i < npositions; i++)
+									for (j = 0; j < npositions; j++)
 									{
-										if (htrpos >= positions[i][0] && htrpos < positions[i][1])
+										if (pos >= positions[j][0] && pos < positions[j][1])
 										{
 											new_attr = t->cursor_pattern_attr;
 											break;
@@ -492,7 +663,7 @@ window_fill(int window_identifier,
 								}
 								else
 								{
-									if (htrpos < lineinfo->start_char + scrdesc->searchterm_char_size)
+									if (pos < lineinfo->start_char + scrdesc->searchterm_char_size)
 										new_attr = t->cursor_pattern_attr;
 								}
 							}
@@ -503,6 +674,7 @@ window_fill(int window_identifier,
 							waddnstr(win, rowstr, bytes);
 							rowstr += bytes;
 							bytes = 0;
+							saved_pos = pos;
 						}
 
 						if (new_attr != active_attr)
@@ -512,6 +684,7 @@ window_fill(int window_identifier,
 								waddnstr(win, rowstr, bytes);
 								rowstr += bytes;
 								bytes = 0;
+								saved_pos = pos;
 							}
 
 							/* disable current style */
@@ -527,6 +700,12 @@ window_fill(int window_identifier,
 							waddch(win, ACS_VLINE);
 							bytes = 0;
 							rowstr += 1;
+
+							/*
+							 * because we printed subst char already and we updated rowstr,
+							 * we don't would to increase bytes variable later.
+							 */
+							skip_char = true;
 						}
 					}
 					else
@@ -535,18 +714,38 @@ window_fill(int window_identifier,
 						{
 							int		new_attr;
 
-							if (is_cursor_row)
+							if (is_cross_cursor)
+							{
+								new_attr = t->cross_cursor_line_attr;
+							}
+							else if (is_cursor)
+							{
 								new_attr = t->cursor_line_attr;
+							}
 							else
+							{
 								new_attr = t->line_attr;
+							}
 
 							if (new_attr != active_attr)
 							{
 								if (bytes > 0)
 								{
-									waddnstr(win, rowstr, bytes);
+									flush_bytes(win,
+												rowstr,
+												bytes,
+												saved_pos,
+												is_top_deco,
+												is_head_deco,
+												is_bottom_deco,
+												desc,
+												scrdesc,
+					  							opts);
+
+
 									rowstr += bytes;
 									bytes = 0;
+									saved_pos = pos;
 								}
 
 								/* disable current style */
@@ -565,102 +764,64 @@ window_fill(int window_identifier,
 						{
 							i += 1;
 							ptr += 1;
-							/* suboptimal logic */
-							if (i <= maxx && rowstr < ptr)
+
+							if (!skip_char)
 								bytes += 1;
 						}
 						else
 						{
+							int dsplen = utf_dsplen(ptr);
 							int len  = utf8charlen(*ptr);
-							i += utf_dsplen(ptr);
+
+							i = dsplen != -1 && i != -1 ? i + dsplen : -1;
 							ptr += len;
-							/* suboptimal logic */
-							if (i <= maxx && rowstr < ptr)
+
+							if (!skip_char)
 								bytes += len;
 						}
 					}
 					else
 					{
+
+						/*
+						 * psql reduces trailing spaces when border is 0 or 1. These spaces
+						 * should be printed after content.
+						 */
+						if (is_vertical_cursor && i != -1)
+						{
+							int ts1 = maxx - i + 1;
+							int ts2 = vcursor_xmax - i + 1;
+
+							trailing_spaces = ts1 < ts2 ? ts1 : ts2;
+						}
+
 						break;
 					}
 				} /* end while */
 			}
-			else if (is_cursor_row)
-				/* in this case i is not valid, but it is necessary for cursor line printing */
-				i = 1;
 
 			if (bytes > 0)
+				flush_bytes(win,
+							rowstr,
+							bytes,
+							saved_pos,
+							is_top_deco,
+							is_head_deco,
+							is_bottom_deco,
+							desc,
+							scrdesc,
+							opts);
+
+			/* print trailing spaces necessary for correct vertical cursor */
+			if (trailing_spaces > 0)
 			{
-				if (!fix_line_attr_style || !(desc->linestyle == 'a' && opts->force_uniborder))
-					waddnstr(win, rowstr, bytes);
-				else
-				{
-					int		i = 0;
-
-					while (bytes > 0)
-					{
-						int		htrpos = srcx + i;
-						int		column_format = desc->headline_transl[htrpos];
-						bool	is_top_row = effective_row == desc->border_top_row;
-						bool	is_head_row = effective_row == desc->border_head_row;
-
-						if (column_format == 'd' && *rowstr == '-')
-						{
-							waddch(win, ACS_HLINE);
-							rowstr += 1;
-							bytes -= 1;
-						}
-						else if (column_format == 'L' && (*rowstr == '+' || *rowstr == '|'))
-						{
-							if (is_head_row)
-								waddch(win, ACS_LTEE);
-							else if (is_top_row)
-								waddch(win, ACS_ULCORNER);
-							else /* bottom row */
-								waddch(win, ACS_LLCORNER);
-
-							rowstr += 1;
-							bytes -= 1;
-						}
-						else if (column_format == 'I' && *rowstr == '+')
-						{
-							if (is_head_row)
-								waddch(win, ACS_PLUS);
-							else if (is_top_row)
-								waddch(win, ACS_TTEE);
-							else /* bottom row */
-								waddch(win, ACS_BTEE);
-
-							rowstr += 1;
-							bytes -= 1;
-						}
-						else if (column_format == 'R' && (*rowstr == '+' || *rowstr == '|'))
-						{
-							if (is_head_row)
-								waddch(win, ACS_RTEE);
-							else if (is_top_row)
-								waddch(win, ACS_URCORNER);
-							else /* bottom row */
-								waddch(win, ACS_LRCORNER);
-
-							rowstr += 1;
-							bytes -= 1;
-
-						}
-						else
-						{
-							int len = opts->force8bit ? 1 : utf8charlen(*rowstr);
-
-							waddnstr(win, rowstr, len);
-							rowstr +=len;
-							bytes -= len;
-						}
-
-						i += 1;
-					}
-				}
+				wprintw(win, "%*s", trailing_spaces, "");
+				i += trailing_spaces;
 			}
 
+			/* When we don't possition of last char, we can (for cursor draw) use 0 */
+			i = i != -1 ? i : 0;
+
 			/* clean other chars on line */
 			if (i < maxx)
 				wclrtoeol(win);
@@ -669,13 +830,13 @@ window_fill(int window_identifier,
 			if (i < maxx)
 			{
 				if (is_cursor_row && !is_bookmark_row)
-					mvwchgat(win, row - 1, i, -1, 0, PAIR_NUMBER(t->cursor_data_attr), 0);
+					mvwchgat(win, row - 1, i, -1, t->cursor_data_attr, PAIR_NUMBER(t->cursor_data_attr), 0);
 				else if (!is_cursor_row && is_bookmark_row)
-					mvwchgat(win, row - 1, i, -1, 0, PAIR_NUMBER(t->bookmark_data_attr), 0);
+					mvwchgat(win, row - 1, i, -1, t->bookmark_data_attr, PAIR_NUMBER(t->bookmark_data_attr), 0);
 				else if (is_cursor_row && is_bookmark_row)
 					mvwchgat(win, row - 1, i, -1, t->cursor_bookmark_attr, PAIR_NUMBER(t->cursor_bookmark_attr), 0);
 				else if (!is_cursor_row && is_pattern_row)
-					mvwchgat(win, row - 1, i, -1, 0, PAIR_NUMBER(t->pattern_data_attr), 0);
+					mvwchgat(win, row - 1, i, -1, t->pattern_data_attr, PAIR_NUMBER(t->pattern_data_attr), 0);
 			}
 
 			if (free_row != NULL)
@@ -773,18 +934,35 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 	{
 		int			bytes;
 		char	   *ptr;
-		char	   *rowstr;
+		char	   *rowstr = NULL;
 
-		if (lnb_row == 1000)
+		if (desc->order_map)
 		{
-			lnb = lnb->next;
-			lnb_row = 0;
-		}
+			int		rowno = row + srcy_bak;
 
-		if (lnb != NULL && lnb_row < lnb->nrows)
-			rowstr = lnb->rows[lnb_row++];
+			if (rowno <= desc->last_row)
+			{
+				MappedLine *mp = &desc->order_map[rowno];
+
+				lnb = mp->lnb;
+				lnb_row = mp->lnb_row;
+				rowstr = lnb->rows[lnb_row];
+			}
+		}
 		else
-			rowstr = NULL;
+		{
+			if (lnb_row == 1000)
+			{
+				lnb = lnb ? lnb->next : NULL;
+				lnb_row = 0;
+			}
+
+			if (lnb != NULL && lnb_row < lnb->nrows)
+			{
+				rowstr = lnb->rows[lnb_row];
+				lnb_row += 1;
+			}
+		}
 
 		active_attr = line_attr;
 		printf("%s", ansi_attr(active_attr));
@@ -857,6 +1035,12 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 				char *p;
 
 				free_row = malloc(left_spaces + strlen(rowstr) + 1);
+				if (!free_row)
+				{
+					fprintf(stderr, "out of memory");
+					exit(1);
+				}
+
 				p = free_row;
 				while (left_spaces-- > 0)
 				{
@@ -877,10 +1061,11 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 				i = 0;
 				while (i < maxx)
 				{
+					int		pos = srcx + i;
+
 					if (is_expand_head)
 					{
-						int		pos = srcx + i;
-						int		new_attr;
+						attr_t	new_attr;
 
 						new_attr = pos >= ei_min && pos <= ei_max ? expi_attr : line_attr;
 
@@ -900,13 +1085,11 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 					}
 					else if (!fix_line_attr_style && desc->headline_transl != NULL)
 					{
-						int htrpos = srcx + i;
-
-						if (htrpos < desc->headline_char_size)
+						if (pos < desc->headline_char_size)
 						{
-							int		new_attr;
+							attr_t	new_attr;
 
-							new_attr = desc->headline_transl[htrpos] == 'd' ? data_attr : line_attr;
+							new_attr = desc->headline_transl[pos] == 'd' ? data_attr : line_attr;
 
 							if (new_attr != active_attr)
 							{
@@ -956,8 +1139,6 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 	}
 }
 
-#define FIXVTE		1
-
 void
 draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		  int first_data_row, int first_row, int cursor_col,
@@ -965,20 +1146,26 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 {
 	struct winsize size;
 	int		i;
-	int		nrows;
 
 	if (ioctl(0, TIOCGWINSZ, (char *) &size) >= 0)
 	{
-		for (i = 0; i < min_int(size.ws_row - 1 - scrdesc->top_bar_rows, desc->last_row + 1); i++)
+		int		expected_rows = min_int(size.ws_row - 1 - scrdesc->top_bar_rows,
+										desc->last_row + 1);
+
+		for (i = 0; i < expected_rows; i++)
 			printf("\eD");
 
 		/* Go wit cursor to up */
-		printf("\e[%dA", min_int(size.ws_row - 1 - scrdesc->top_bar_rows, desc->last_row + 1));
+		printf("\e[%dA", expected_rows);
+
+		/* now, the screen can be different, due prompt */
+		scrdesc->rows_rows = min_int(scrdesc->rows_rows,
+									 expected_rows - scrdesc->fix_rows_rows);
 
 		/*
 		 * Save cursor - Attention, there are a Fedora29 bug, and it doesn't work
 		 */
-		printf("\e[s");
+		printf("\e7");
 
 		if (scrdesc->fix_cols_cols > 0)
 		{
@@ -988,16 +1175,12 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
-			nrows = scrdesc->fix_rows_rows + scrdesc->rows_rows;
 		}
+
 		if (scrdesc->fix_rows_rows > 0 )
 		{
-			if (FIXVTE)
-				/* Go wit cursor to up */
-				printf("\e[%dA", nrows);
-			else
-				/* Go to saved position */
-				printf("\e[u\e[s");
+			/* Go to saved position */
+			printf("\e8\e7");
 
 			draw_rectange(0, scrdesc->fix_cols_cols,
 						  scrdesc->fix_rows_rows, size.ws_col - scrdesc->fix_cols_cols,
@@ -1005,17 +1188,12 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  true);
-			nrows = 0 + scrdesc->fix_rows_rows;
 		}
 
 		if (scrdesc->fix_rows_rows > 0 && scrdesc->fix_cols_cols > 0)
 		{
-			if (FIXVTE)
-				/* Go wit cursor to up */
-				printf("\e[%dA", nrows);
-			else
-				/* Go to saved position */
-				printf("\e[u\e[s");
+			/* Go to saved position */
+			printf("\e8\e7");
 
 			draw_rectange(0, 0,
 						  scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
@@ -1023,18 +1201,12 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
-
-			nrows = 0 + scrdesc->fix_rows_rows;
 		}
 
 		if (scrdesc->rows_rows > 0)
 		{
-			if (FIXVTE)
-				/* Go wit cursor to up */
-				printf("\e[%dA", nrows);
-			else
-				/* Go to saved position */
-				printf("\e[u\e[s");
+			/* Go to saved position */
+			printf("\e8\e7");
 
 			draw_rectange(scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  scrdesc->rows_rows, size.ws_col - scrdesc->fix_cols_cols,
@@ -1048,12 +1220,8 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 
 		if (w_footer(scrdesc) != NULL)
 		{
-			if (FIXVTE)
-				/* Go wit cursor to up */
-				printf("\e[%dA", nrows);
-			else
-				/* Go to saved position */
-				printf("\e[u\e[s");
+			/* Go to saved position */
+			printf("\e8\e7");
 
 			draw_rectange(scrdesc->fix_rows_rows + scrdesc->rows_rows, 0,
 						  scrdesc->footer_rows, scrdesc->maxx,
