@@ -3,7 +3,7 @@
  * themes.h
  *	  themes initialization
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/themes.h
@@ -51,6 +51,12 @@ typedef struct
 	attr_t bottom_light_attr;		/* colors for lighter bottom text */
 	attr_t prompt_attr;				/* less prompt color */
 	attr_t cursor_rownum_attr;		/* colors for cursor rownum column */
+	attr_t cross_cursor_attr;		/* colors for cross horizontal and vertical cursor (data) */
+	attr_t cross_cursor_line_attr;	/* colors for cross horizontal and vertical cursor (border) */
+	attr_t pattern_vertical_cursor_attr;
+	attr_t pattern_vertical_cursor_line_attr;
+	attr_t error_attr;				/* colors for display error */
+	attr_t input_attr;				/* colors for input line */
 } Theme;
 
 #define		WINDOW_LUC				0
