@@ -3,7 +3,7 @@
  * unicode.c
  *	  unicode and wide chars routines
  *
- * Portions Copyright (c) 2017-2018 Pavel Stehule
+ * Portions Copyright (c) 2017-2019 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/unicode.c
@@ -11,6 +11,9 @@
  *-------------------------------------------------------------------------
  */
 
+#include <ctype.h>
+#include <stdbool.h>
+
 #include "unicode.h"
 #include "string.h"
 
@@ -45,7 +48,7 @@ utf8len_start_stop(const char *start, const char *stop)
 /*
  * Returns length of utf8 char in bytes
  */
-int
+inline int
 utf8charlen(char ch)
 {
 	if ((ch & 0x80) == 0)
@@ -139,40 +142,70 @@ ucs_wcwidth(wchar_t ucs)
 {
 	/* sorted list of non-overlapping intervals of non-spacing characters */
 	static const struct mbinterval combining[] = {
-		{0x0300, 0x034E}, {0x0360, 0x0362}, {0x0483, 0x0486},
-		{0x0488, 0x0489}, {0x0591, 0x05A1}, {0x05A3, 0x05B9},
-		{0x05BB, 0x05BD}, {0x05BF, 0x05BF}, {0x05C1, 0x05C2},
-		{0x05C4, 0x05C4}, {0x064B, 0x0655}, {0x0670, 0x0670},
-		{0x06D6, 0x06E4}, {0x06E7, 0x06E8}, {0x06EA, 0x06ED},
-		{0x070F, 0x070F}, {0x0711, 0x0711}, {0x0730, 0x074A},
-		{0x07A6, 0x07B0}, {0x0901, 0x0902}, {0x093C, 0x093C},
-		{0x0941, 0x0948}, {0x094D, 0x094D}, {0x0951, 0x0954},
-		{0x0962, 0x0963}, {0x0981, 0x0981}, {0x09BC, 0x09BC},
-		{0x09C1, 0x09C4}, {0x09CD, 0x09CD}, {0x09E2, 0x09E3},
-		{0x0A02, 0x0A02}, {0x0A3C, 0x0A3C}, {0x0A41, 0x0A42},
-		{0x0A47, 0x0A48}, {0x0A4B, 0x0A4D}, {0x0A70, 0x0A71},
-		{0x0A81, 0x0A82}, {0x0ABC, 0x0ABC}, {0x0AC1, 0x0AC5},
-		{0x0AC7, 0x0AC8}, {0x0ACD, 0x0ACD}, {0x0B01, 0x0B01},
-		{0x0B3C, 0x0B3C}, {0x0B3F, 0x0B3F}, {0x0B41, 0x0B43},
-		{0x0B4D, 0x0B4D}, {0x0B56, 0x0B56}, {0x0B82, 0x0B82},
-		{0x0BC0, 0x0BC0}, {0x0BCD, 0x0BCD}, {0x0C3E, 0x0C40},
-		{0x0C46, 0x0C48}, {0x0C4A, 0x0C4D}, {0x0C55, 0x0C56},
+		{0x0300, 0x036F}, {0x0483, 0x0489}, {0x0591, 0x05BD},
+		{0x05BF, 0x05BF}, {0x05C1, 0x05C2}, {0x05C4, 0x05C5},
+		{0x05C7, 0x05C7}, {0x0610, 0x061A}, {0x064B, 0x065F},
+		{0x0670, 0x0670}, {0x06D6, 0x06DC}, {0x06DF, 0x06E4},
+		{0x06E7, 0x06E8}, {0x06EA, 0x06ED}, {0x0711, 0x0711},
+		{0x0730, 0x074A}, {0x07A6, 0x07B0}, {0x07EB, 0x07F3},
+		{0x07FD, 0x07FD}, {0x0816, 0x0819}, {0x081B, 0x0823},
+		{0x0825, 0x0827}, {0x0829, 0x082D}, {0x0859, 0x085B},
+		{0x08D3, 0x08E1}, {0x08E3, 0x0902}, {0x093A, 0x093A},
+		{0x093C, 0x093C}, {0x0941, 0x0948}, {0x094D, 0x094D},
+		{0x0951, 0x0957}, {0x0962, 0x0963}, {0x0981, 0x0981},
+		{0x09BC, 0x09BC}, {0x09C1, 0x09C4}, {0x09CD, 0x09CD},
+		{0x09E2, 0x09E3}, {0x09FE, 0x0A02}, {0x0A3C, 0x0A3C},
+		{0x0A41, 0x0A51}, {0x0A70, 0x0A71}, {0x0A75, 0x0A75},
+		{0x0A81, 0x0A82}, {0x0ABC, 0x0ABC}, {0x0AC1, 0x0AC8},
+		{0x0ACD, 0x0ACD}, {0x0AE2, 0x0AE3}, {0x0AFA, 0x0B01},
+		{0x0B3C, 0x0B3C}, {0x0B3F, 0x0B3F}, {0x0B41, 0x0B44},
+		{0x0B4D, 0x0B56}, {0x0B62, 0x0B63}, {0x0B82, 0x0B82},
+		{0x0BC0, 0x0BC0}, {0x0BCD, 0x0BCD}, {0x0C00, 0x0C00},
+		{0x0C04, 0x0C04}, {0x0C3E, 0x0C40}, {0x0C46, 0x0C56},
+		{0x0C62, 0x0C63}, {0x0C81, 0x0C81}, {0x0CBC, 0x0CBC},
 		{0x0CBF, 0x0CBF}, {0x0CC6, 0x0CC6}, {0x0CCC, 0x0CCD},
-		{0x0D41, 0x0D43}, {0x0D4D, 0x0D4D}, {0x0DCA, 0x0DCA},
-		{0x0DD2, 0x0DD4}, {0x0DD6, 0x0DD6}, {0x0E31, 0x0E31},
+		{0x0CE2, 0x0CE3}, {0x0D00, 0x0D01}, {0x0D3B, 0x0D3C},
+		{0x0D41, 0x0D44}, {0x0D4D, 0x0D4D}, {0x0D62, 0x0D63},
+		{0x0DCA, 0x0DCA}, {0x0DD2, 0x0DD6}, {0x0E31, 0x0E31},
 		{0x0E34, 0x0E3A}, {0x0E47, 0x0E4E}, {0x0EB1, 0x0EB1},
-		{0x0EB4, 0x0EB9}, {0x0EBB, 0x0EBC}, {0x0EC8, 0x0ECD},
-		{0x0F18, 0x0F19}, {0x0F35, 0x0F35}, {0x0F37, 0x0F37},
-		{0x0F39, 0x0F39}, {0x0F71, 0x0F7E}, {0x0F80, 0x0F84},
-		{0x0F86, 0x0F87}, {0x0F90, 0x0F97}, {0x0F99, 0x0FBC},
-		{0x0FC6, 0x0FC6}, {0x102D, 0x1030}, {0x1032, 0x1032},
-		{0x1036, 0x1037}, {0x1039, 0x1039}, {0x1058, 0x1059},
-		{0x1160, 0x11FF}, {0x17B7, 0x17BD}, {0x17C6, 0x17C6},
-		{0x17C9, 0x17D3}, {0x180B, 0x180E}, {0x18A9, 0x18A9},
-		{0x200B, 0x200F}, {0x202A, 0x202E}, {0x206A, 0x206F},
-		{0x20D0, 0x20E3}, {0x302A, 0x302F}, {0x3099, 0x309A},
-		{0xFB1E, 0xFB1E}, {0xFE20, 0xFE23}, {0xFEFF, 0xFEFF},
-		{0xFFF9, 0xFFFB}
+		{0x0EB4, 0x0EBC}, {0x0EC8, 0x0ECD}, {0x0F18, 0x0F19},
+		{0x0F35, 0x0F35}, {0x0F37, 0x0F37}, {0x0F39, 0x0F39},
+		{0x0F71, 0x0F7E}, {0x0F80, 0x0F84}, {0x0F86, 0x0F87},
+		{0x0F8D, 0x0FBC}, {0x0FC6, 0x0FC6}, {0x102D, 0x1030},
+		{0x1032, 0x1037}, {0x1039, 0x103A}, {0x103D, 0x103E},
+		{0x1058, 0x1059}, {0x105E, 0x1060}, {0x1071, 0x1074},
+		{0x1082, 0x1082}, {0x1085, 0x1086}, {0x108D, 0x108D},
+		{0x109D, 0x109D}, {0x135D, 0x135F}, {0x1712, 0x1714},
+		{0x1732, 0x1734}, {0x1752, 0x1753}, {0x1772, 0x1773},
+		{0x17B4, 0x17B5}, {0x17B7, 0x17BD}, {0x17C6, 0x17C6},
+		{0x17C9, 0x17D3}, {0x17DD, 0x17DD}, {0x180B, 0x180D},
+		{0x1885, 0x1886}, {0x18A9, 0x18A9}, {0x1920, 0x1922},
+		{0x1927, 0x1928}, {0x1932, 0x1932}, {0x1939, 0x193B},
+		{0x1A17, 0x1A18}, {0x1A1B, 0x1A1B}, {0x1A56, 0x1A56},
+		{0x1A58, 0x1A60}, {0x1A62, 0x1A62}, {0x1A65, 0x1A6C},
+		{0x1A73, 0x1A7F}, {0x1AB0, 0x1B03}, {0x1B34, 0x1B34},
+		{0x1B36, 0x1B3A}, {0x1B3C, 0x1B3C}, {0x1B42, 0x1B42},
+		{0x1B6B, 0x1B73}, {0x1B80, 0x1B81}, {0x1BA2, 0x1BA5},
+		{0x1BA8, 0x1BA9}, {0x1BAB, 0x1BAD}, {0x1BE6, 0x1BE6},
+		{0x1BE8, 0x1BE9}, {0x1BED, 0x1BED}, {0x1BEF, 0x1BF1},
+		{0x1C2C, 0x1C33}, {0x1C36, 0x1C37}, {0x1CD0, 0x1CD2},
+		{0x1CD4, 0x1CE0}, {0x1CE2, 0x1CE8}, {0x1CED, 0x1CED},
+		{0x1CF4, 0x1CF4}, {0x1CF8, 0x1CF9}, {0x1DC0, 0x1DFF},
+		{0x20D0, 0x20F0}, {0x2CEF, 0x2CF1}, {0x2D7F, 0x2D7F},
+		{0x2DE0, 0x2DFF}, {0x302A, 0x302D}, {0x3099, 0x309A},
+		{0xA66F, 0xA672}, {0xA674, 0xA67D}, {0xA69E, 0xA69F},
+		{0xA6F0, 0xA6F1}, {0xA802, 0xA802}, {0xA806, 0xA806},
+		{0xA80B, 0xA80B}, {0xA825, 0xA826}, {0xA8C4, 0xA8C5},
+		{0xA8E0, 0xA8F1}, {0xA8FF, 0xA8FF}, {0xA926, 0xA92D},
+		{0xA947, 0xA951}, {0xA980, 0xA982}, {0xA9B3, 0xA9B3},
+		{0xA9B6, 0xA9B9}, {0xA9BC, 0xA9BD}, {0xA9E5, 0xA9E5},
+		{0xAA29, 0xAA2E}, {0xAA31, 0xAA32}, {0xAA35, 0xAA36},
+		{0xAA43, 0xAA43}, {0xAA4C, 0xAA4C}, {0xAA7C, 0xAA7C},
+		{0xAAB0, 0xAAB0}, {0xAAB2, 0xAAB4}, {0xAAB7, 0xAAB8},
+		{0xAABE, 0xAABF}, {0xAAC1, 0xAAC1}, {0xAAEC, 0xAAED},
+		{0xAAF6, 0xAAF6}, {0xABE5, 0xABE5}, {0xABE8, 0xABE8},
+		{0xABED, 0xABED}, {0xFB1E, 0xFB1E}, {0xFE00, 0xFE0F},
+		{0xFE20, 0xFE2F},
 	};
 
 	/* test for 8-bit control characters */
@@ -273,12 +306,32 @@ utf8_to_unicode(const unsigned char *c)
 						   ((c[2] & 0x3f) << 6) |
 						   (c[3] & 0x3f));
 	else
+
+#if __WCHAR_MAX__ > 0x10000
+
 		/* that is an invalid code on purpose */
 		return 0xffffffff;
+
+#else
+
+		/* that is an invalid code on purpose */
+		return 0xffff;
+
+#endif
+
 }
 
-int
+inline int
 utf_dsplen(const char *s)
+{
+	if (*s == ' ')
+		return 1;
+
+	return ucs_wcwidth(utf8_to_unicode((const unsigned char *) s));
+}
+
+inline static int
+_utf_dsplen(const char *s)
 {
 	return ucs_wcwidth(utf8_to_unicode((const unsigned char *) s));
 }
@@ -295,16 +348,73 @@ utf_string_dsplen(const char *s, size_t max_bytes)
 
 	while (*ptr != '\0' && max_bytes > 0)
 	{
-		int		clen = utf8charlen(*ptr);
+		if (*ptr == ' ')
+		{
+			ptr += 1;
+			result += 1;
+			max_bytes -= 1;
+		} 
+		else
+		{
+			int		clen;
 
-		result += utf_dsplen(ptr);
-		ptr += clen;
-		max_bytes -= clen;
+			clen  = utf8charlen(*ptr);
+			result += _utf_dsplen(ptr);
+			ptr += clen;
+			max_bytes -= clen;
+		}
 	}
 
 	return result;
 }
 
+int
+utf_string_dsplen_multiline(const char *s, size_t max_bytes, bool *multiline, bool first_only, long int *digits, long int *others)
+{
+	int result = -1;
+	int		rowlen = 0;
+	const char *ptr = s;
+
+	*multiline = false;
+
+	while (*ptr != '\0' && max_bytes > 0)
+	{
+		int		clen;
+
+		if (!first_only)
+		{
+			if (*ptr >= '0' && *ptr <= '9')
+				(*digits)++;
+			else if (*ptr != '-' && *ptr != ' ' && *ptr != ':')
+				(*others)++;
+		}
+
+		if (*ptr == '\n')
+		{
+			*multiline = true;
+
+			result = rowlen > result ? rowlen : result;
+			max_bytes -= 1;
+
+			rowlen = 0;
+			ptr += 1;
+
+			if (first_only)
+				break;
+
+			continue;
+		}
+
+		clen = utf8charlen(*ptr);
+		rowlen += utf_dsplen(ptr);
+		ptr += clen;
+		max_bytes -= clen;
+	}
+
+	return result != -1 ? result : rowlen;
+}
+
+
 /*
  * This version of previous function uses similar calculation like
  * readline libarry - and supports terminal tabs.
@@ -341,16 +451,16 @@ readline_utf_string_dsplen(const char *s, size_t max_bytes, size_t offset)
  */
 typedef struct conv_table
 {
-    unsigned int first;
-    unsigned int last;
+    wchar_t first;
+    wchar_t last;
     int step;
     int offset;
 } conv_table;
 
 typedef struct range_table
 {
-    unsigned int first;
-    unsigned int last;
+    wchar_t first;
+    wchar_t last;
     int step;
 } range_table;
 
@@ -463,14 +573,67 @@ utf8_tofold(const char *s)
 		{ 0xA796, 0xA7A8, 2, 1 }, { 0xA7AA, 0xA7AA, 1, -42308 }, { 0xA7AB, 0xA7AB, 1, -42319 },
 		{ 0xA7AC, 0xA7AC, 1, -42315 },{ 0xA7AD, 0xA7AD, 1, -42305 }, { 0xA7B0, 0xA7B0, 1, -42258 },
 		{ 0xA7B1, 0xA7B1, 1, -42282 }, { 0xA7B2, 0xA7B2, 1, -42261 }, { 0xA7B3, 0xA7B3, 1, 928 },
-		{ 0xA7B4, 0xA7B6, 2, 1 }, { 0xAB70, 0xABBF, 1, -38864 }, { 0xFF21, 0xFF3A, 1, 32 },
-		{ 0x10400, 0x10427, 1, 40 }, { 0x10C80, 0x10CB2, 1, 64 }, { 0x118A0, 0x118BF, 1, 32 }
+		{ 0xA7B4, 0xA7B6, 2, 1 }, { 0xAB70, 0xABBF, 1, -38864 }, { 0xFF21, 0xFF3A, 1, 32 }
+
+#if __WCHAR_MAX__ > 0x10000
+
+		, { 0x10400, 0x10427, 1, 40 }, { 0x10C80, 0x10CB2, 1, 64 }, { 0x118A0, 0x118BF, 1, 32 }
+
+#endif
+
 	};
 
 	return convert_char(tofold_table, table_size(tofold_table),
 						  utf8_to_unicode((const unsigned char *) s));
 }
 
+const char *
+utf8_nstrstr_with_sizes(const char *haystack,
+						int haystack_size,
+						const char *needle,
+						int needle_size)
+{
+	const char *haystack_cur, *needle_cur, *needle_prev;
+	const char *haystack_end, *needle_end;
+	int		f1 = 0, f2 = 0;
+	int		needle_char_len = 0; /* be compiler quiet */
+
+	needle_cur = needle;
+	needle_prev = NULL;
+	haystack_cur = haystack;
+
+	haystack_end = haystack + haystack_size;
+	needle_end = needle + needle_size;
+
+	while (needle_cur < needle_end)
+	{
+		if (haystack_cur == haystack_end)
+			return NULL;
+
+		if (needle_prev != needle_cur)
+		{
+			needle_prev = needle_cur;
+			needle_char_len = utf8charlen(*needle_cur);
+			f1 = utf8_tofold(needle_cur);
+		}
+
+		f2 = utf8_tofold(haystack_cur);
+		if (f1 == f2)
+		{
+			needle_cur += needle_char_len;
+			haystack_cur += utf8charlen(*haystack_cur);
+		}
+		else
+		{
+			needle_cur = needle;
+			haystack_cur = haystack += utf8charlen(*haystack);
+		}
+	}
+
+	return haystack;
+}
+
+
 const char *
 utf8_nstrstr(const char *haystack, const char *needle)
 {
@@ -613,7 +776,11 @@ utf8_isupper(const char *s)
 		{ 0xA732, 0xA76E, 2 }, { 0xA779, 0xA77D, 2 }, { 0xA77E, 0xA786, 2 },
 		{ 0xA78B, 0xA78D, 2 }, { 0xA790, 0xA792, 2 }, { 0xA796, 0xA7AA, 2 },
 		{ 0xA7AB, 0xA7AD, 1 }, { 0xA7B0, 0xA7B4, 1 }, { 0xA7B6, 0xFF21, 22379 },
-		{ 0xFF22, 0xFF3A, 1 }, { 0x10400, 0x10427, 1 }, { 0x10C80, 0x10CB2, 1 },
+		{ 0xFF22, 0xFF3A, 1 }
+
+#if __WCHAR_MAX__ > 0x10000
+
+							,    { 0x10400, 0x10427, 1 }, { 0x10C80, 0x10CB2, 1 },
 		{ 0x118A0, 0x118BF, 1 }, { 0x1D400, 0x1D419, 1 }, { 0x1D434, 0x1D44D, 1 },
 		{ 0x1D468, 0x1D481, 1 }, { 0x1D49C, 0x1D49E, 2 }, { 0x1D49F, 0x1D4A5, 3 },
 		{ 0x1D4A6, 0x1D4A9, 3 }, { 0x1D4AA, 0x1D4AC, 1 }, { 0x1D4AE, 0x1D4B5, 1 },
@@ -626,8 +793,70 @@ utf8_isupper(const char *s)
 		{ 0x1D71C, 0x1D734, 1 }, { 0x1D756, 0x1D76E, 1 }, { 0x1D790, 0x1D7A8, 1 },
 		{ 0x1D7CA, 0x1F130, 6502 }, { 0x1F131, 0x1F149, 1 }, { 0x1F150, 0x1F169, 1 },
 		{ 0x1F170, 0x1F189, 1 },
+
+#endif
+
 	};
 
 	return find_in_range(upper_table, table_size(upper_table),
 						utf8_to_unicode((const unsigned char *) s)) != 0;
 }
+
+int
+utf2wchar_with_len(const unsigned char *from, wchar_t *to, int len)
+{
+	int			cnt = 0;
+	unsigned int	c1,
+					c2,
+					c3,
+					c4;
+
+	while (len > 0 && *from)
+	{
+		if ((*from & 0x80) == 0)
+		{
+			*to = *from++;
+			len--;
+		}
+		else if ((*from & 0xe0) == 0xc0)
+		{
+			if (len < 2)
+				break;			/* drop trailing incomplete char */
+			c1 = *from++ & 0x1f;
+			c2 = *from++ & 0x3f;
+			*to = (c1 << 6) | c2;
+			len -= 2;
+		}
+		else if ((*from & 0xf0) == 0xe0)
+		{
+			if (len < 3)
+				break;			/* drop trailing incomplete char */
+			c1 = *from++ & 0x0f;
+			c2 = *from++ & 0x3f;
+			c3 = *from++ & 0x3f;
+			*to = (c1 << 12) | (c2 << 6) | c3;
+			len -= 3;
+		}
+		else if ((*from & 0xf8) == 0xf0)
+		{
+			if (len < 4)
+				break;			/* drop trailing incomplete char */
+			c1 = *from++ & 0x07;
+			c2 = *from++ & 0x3f;
+			c3 = *from++ & 0x3f;
+			c4 = *from++ & 0x3f;
+			*to = (c1 << 18) | (c2 << 12) | (c3 << 6) | c4;
+			len -= 4;
+		}
+		else
+		{
+			/* treat a bogus char as length 1; not ours to raise error */
+			*to = *from++;
+			len--;
+		}
+		to++;
+		cnt++;
+	}
+	*to = 0;
+	return cnt;
+}
